@@ -41,6 +41,10 @@
 #import "OFRunLoop+Private.h"
 #import "OFRunLoop.h"
 #import "OFSandbox.h"
+#ifdef OF_HAVE_SOCKETS
+# import "OFSocket.h"
+# import "OFSocket+Private.h"
+#endif
 #import "OFStdIOStream.h"
 #import "OFString.h"
 #import "OFSystemInfo.h"
@@ -56,8 +60,6 @@
 # include <crt_externs.h>
 #elif defined(OF_WINDOWS)
 # include <windows.h>
-extern int _CRT_glob;
-extern void __wgetmainargs(int *, wchar_t ***, wchar_t ***, int, int *);
 #elif defined(OF_AMIGAOS)
 # define Class IntuitionClass
 # include <proto/exec.h>
@@ -114,7 +116,7 @@ atexitHandler(void)
 
 #if defined(OF_HAVE_THREADS) && defined(OF_HAVE_SOCKETS) && \
     defined(OF_AMIGAOS) && !defined(OF_MORPHOS)
-	OFSocketDeinit();
+	_OFSocketDeinit();
 #endif
 }
 
@@ -127,6 +129,9 @@ OFApplicationMain(int *argc, char **argv[], id <OFApplicationDelegate> delegate)
 
 #ifdef OF_WINDOWS
 	if ([OFSystemInfo isWindowsNT]) {
+		extern void __wgetmainargs(int *, wchar_t ***, wchar_t ***, int,
+		    int *);
+		extern int _CRT_glob;
 		wchar_t **wargv, **wenvp;
 		int wargc, si = 0;
 
