@@ -45,9 +45,6 @@
 #ifdef OF_HAIKU
 # include <kernel/fs_attr.h>
 #endif
-#ifdef OF_WINDOWS
-# include <utime.h>
-#endif
 #ifdef OF_DJGPP
 # include <syslimits.h>
 #endif
@@ -151,7 +148,6 @@ releaseReaddirMutex(void)
 #endif
 
 #ifdef OF_WINDOWS
-static int (*_wutime64FuncPtr)(const wchar_t *, struct __utimbuf64 *);
 static WINAPI BOOLEAN (*createSymbolicLinkWFuncPtr)(LPCWSTR, LPCWSTR, DWORD);
 static WINAPI BOOLEAN (*createHardLinkWFuncPtr)(LPCWSTR, LPCWSTR,
     LPSECURITY_ATTRIBUTES);
@@ -170,6 +166,19 @@ filetimeToTimeInterval(const FILETIME *filetime)
 	    filetime->dwLowDateTime) / 10000000.0 - 11644473600.0;
 }
 
+static FILETIME
+timeIntervalToFiletime(OFTimeInterval timeInterval)
+{
+	uint64_t timestamp =
+	    (uint64_t)((timeInterval + 11644473600.0) * 10000000.0);
+	FILETIME filetime = {
+		.dwHighDateTime = timestamp >> 32,
+		.dwLowDateTime = timestamp & 0xFFFFFFFF
+	};
+
+	return filetime;
+}
+
 static int
 lastError(void)
 {
@@ -754,6 +763,62 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		fs_close_attr_dir(dir);
 	}
+# elif defined(OF_SOLARIS)
+	int fd;
+	DIR *dir;
+
+	if ((fd = attropen(cPath, ".", O_RDONLY)) == -1)
+		return;
+
+	if ((dir = fdopendir(fd)) == NULL) {
+		close(fd);
+		return;
+	}
+
+#  ifdef OF_HAVE_THREADS
+	@try {
+		[readdirMutex lock];
+	} @catch (id e) {
+		closedir(dir);
+		close(fd);
+		@throw e;
+	}
+#  endif
+
+	@try {
+		names = [OFMutableArray array];
+
+		for (;;) {
+			struct dirent *dirent;
+			OFString *name;
+
+			errno = 0;
+			if ((dirent = readdir(dir)) == NULL) {
+				if (errno == 0)
+					break;
+				else
+					return;
+			}
+
+			if (strcmp(dirent->d_name, ".") == 0 ||
+			    strcmp(dirent->d_name, "..") == 0)
+				continue;
+
+			name = [[OFString alloc] initWithCString: dirent->d_name
+							encoding: encoding];
+			@try {
+				[names addObject: name];
+			} @finally {
+				[name release];
+			}
+		}
+	} @finally {
+#  ifdef OF_HAVE_THREADS
+		[readdirMutex unlock];
+#  endif
+		closedir(dir);
+		close(fd);
+	}
 # endif
 
 	[names makeImmutable];
@@ -781,10 +846,6 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 #endif
 
 #ifdef OF_WINDOWS
-	if ((module = GetModuleHandle("msvcrt.dll")) != NULL)
-		_wutime64FuncPtr = (int (*)(const wchar_t *,
-		    struct __utimbuf64 *))GetProcAddress(module, "_wutime64");
-
 	if ((module = GetModuleHandleA("kernel32.dll")) != NULL) {
 		createSymbolicLinkWFuncPtr =
 		    (WINAPI BOOLEAN (*)(LPCWSTR, LPCWSTR, DWORD))
@@ -899,54 +960,42 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		modificationDate = lastAccessDate;
 
 #if defined(OF_WINDOWS)
-	if (_wutime64FuncPtr != NULL) {
-		struct __utimbuf64 times = {
-			.actime =
-			    (__time64_t)lastAccessDate.timeIntervalSince1970,
-			.modtime =
-			    (__time64_t)modificationDate.timeIntervalSince1970
-		};
+	FILETIME accessTime = timeIntervalToFiletime(
+	    lastAccessDate.timeIntervalSince1970);
+	FILETIME modificationTime = timeIntervalToFiletime(
+	    modificationDate.timeIntervalSince1970);
+	HANDLE handle;
 
-		if (_wutime64FuncPtr([path UTF16String], &times) != 0) {
-			int errNo = errno;
+	if ([OFSystemInfo isWindowsNT])
+		handle = CreateFileW(path.UTF16String, FILE_WRITE_ATTRIBUTES,
+		    FILE_SHARE_READ, NULL, OPEN_EXISTING,
+		    FILE_ATTRIBUTE_NORMAL, NULL);
+	else
+		handle = CreateFileA(
+		    [path cStringWithEncoding: [OFLocale encoding]],
+		    FILE_WRITE_ATTRIBUTES, FILE_SHARE_READ, NULL,
+		    OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 
-			if (errNo == EACCES && [self directoryExistsAtIRI: IRI])
-				errNo = EISDIR;
+	if (handle == NULL)
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: attributeKey
+			       errNo: lastError()];
 
-			@throw [OFSetItemAttributesFailedException
-			    exceptionWithIRI: IRI
-				  attributes: attributes
-			     failedAttribute: attributeKey
-				       errNo: errNo];
-		}
-	} else {
-		struct _utimbuf times = {
-			.actime = (time_t)lastAccessDate.timeIntervalSince1970,
-			.modtime =
-			    (time_t)modificationDate.timeIntervalSince1970
-		};
-		int status;
+	if (!SetFileTime(handle, NULL, &accessTime, &modificationTime)) {
+		int errNo = lastError();
 
-		if ([OFSystemInfo isWindowsNT])
-			status = _wutime([path UTF16String], &times);
-		else
-			status = _utime(
-			    [path cStringWithEncoding: [OFLocale encoding]],
-			    &times);
+		CloseHandle(handle);
 
-		if (status != 0) {
-			int errNo = errno;
-
-			if (errNo == EACCES && [self directoryExistsAtIRI: IRI])
-				errNo = EISDIR;
-
-			@throw [OFSetItemAttributesFailedException
-			    exceptionWithIRI: IRI
-				  attributes: attributes
-			     failedAttribute: attributeKey
-				       errNo: errNo];
-		}
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: attributeKey
+			       errNo: errNo];
 	}
+
+	CloseHandle(handle);
 #elif defined(OF_AMIGAOS)
 	/* AmigaOS does not support access time. */
 	OFTimeInterval modificationTime =
@@ -1717,9 +1766,9 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	OFString *path = IRI.fileSystemRepresentation;
 	OFStringEncoding encoding = [OFLocale encoding];
 	const char *cPath = [path cStringWithEncoding: encoding];
-	void *value = NULL;
 # if defined(OF_LINUX) || defined(OF_MACOS)
 	const char *cName = [name cStringWithEncoding: encoding];
+	void *value = NULL;
 #  if defined(OF_LINUX)
 	ssize_t size = lgetxattr(cPath, cName, NULL, 0);
 #  elif defined(OF_MACOS)
@@ -1757,6 +1806,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	int namespace;
 	const char *cName;
 	ssize_t size;
+	void *value = NULL;
 
 	parseAttributeName(&name, &namespace);
 	cName = [name cStringWithEncoding: encoding];
@@ -1788,6 +1838,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	const char *cName = [name cStringWithEncoding: encoding];
 	int fd = open(cPath, O_RDONLY);
 	struct attr_info info;
+	void *value = NULL;
 
 	if (fd == -1)
 		@throw [OFGetItemAttributesFailedException
@@ -1823,6 +1874,36 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		OFFreeMemory(value);
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName = [name cStringWithEncoding: encoding];
+	int fd;
+
+	if ((fd = attropen(cPath, cName, O_RDONLY)) == -1)
+		@throw [OFGetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			       errNo: errno];
+
+	@try {
+		OFMutableData *mutableData = [OFMutableData data];
+		char buffer[512];
+		ssize_t length;
+
+		while ((length = read(fd, buffer, 512)) > 0)
+			[mutableData addItems: buffer count: length];
+
+		if (length < 0)
+			@throw [OFGetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				       errNo: errno];
+
+		[mutableData makeImmutable];
+		*data = mutableData;
+	} @finally {
+		close(fd);
+	}
+
+	if (type != NULL)
+		*type = nil;
 # endif
 
 	[*data retain];
@@ -1848,12 +1929,13 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	size_t size = data.count * data.itemSize;
 
 # if defined(OF_LINUX) || defined(OF_MACOS)
-	const char *cName = [name cStringWithEncoding: encoding];
+	const char *cName;
 
 	if (type != nil)
 		@throw [OFNotImplementedException exceptionWithSelector: _cmd
 								 object: self];
 
+	cName = [name cStringWithEncoding: encoding];
 #  if defined(OF_LINUX)
 	if (lsetxattr(cPath, cName, data.items, size, 0) != 0) {
 #  elif defined(OF_MACOS)
@@ -1937,6 +2019,39 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName;
+	int fd;
+
+	if (type != nil)
+		@throw [OFNotImplementedException exceptionWithSelector: _cmd
+								 object: self];
+
+	cName = [name cStringWithEncoding: encoding];
+	fd = attropen(cPath, cName, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+
+	if (fd == -1)
+		/* TODO: Add an attribute (prefix?) for extended attributes? */
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: [OFDictionary dictionary]
+		     failedAttribute: @""
+			       errNo: errno];
+
+	@try {
+		if (write(fd, data.items, size) != (ssize_t)size)
+			/*
+			 * TODO: Add an attribute (prefix?) for extended
+			 *	 attributes?
+			 */
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: [OFDictionary dictionary]
+			     failedAttribute: @""
+				       errNo: errno];
+	} @finally {
+		close(fd);
+	}
 # endif
 
 	objc_autoreleasePoolPop(pool);
@@ -2015,6 +2130,32 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	} @finally {
 		close(fd);
 	}
+# elif defined(OF_SOLARIS)
+	const char *cName = [name cStringWithEncoding: encoding];
+	int fd;
+
+	if ((fd = attropen(cPath, ".", O_RDONLY)) < 0)
+		/* TODO: Add an attribute (prefix?) for extended attributes? */
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: [OFDictionary dictionary]
+		     failedAttribute: @""
+			       errNo: errno];
+
+	@try {
+		if (unlinkat(fd, cName, 0) != 0)
+			/*
+			 * TODO: Add an attribute (prefix?) for extended
+			 *	 attributes?
+			 */
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: [OFDictionary dictionary]
+			     failedAttribute: @""
+				       errNo: errno];
+	} @finally {
+		close(fd);
+	}
 # endif
 
 	objc_autoreleasePoolPop(pool);
